﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Microsoft.WindowsAPICodePack.Dialogs;

namespace Perfusjonssytem
{
    //public delegate void Del(int currentIndex);

    public partial class Form1 : Form
    {
        // GUI constrols
        List<ComboBox> cbSubstanceList;


        List<PAction> actionList;
        List<Channel> seqChannels;
        List<Channel> manChannels;
        Sequence currentSequence;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            cbSubstanceList = new List<ComboBox>();
            cbSubstanceList.Add(cbCh1Substance);
            cbSubstanceList.Add(cbCh2Substance);
            cbSubstanceList.Add(cbCh3Substance);
            cbSubstanceList.Add(cbCh4Substance);

            actionList = new List<PAction>();
            seqChannels = new List<Channel>();
            manChannels = new List<Channel>();

            COM.Connected += ConnectionIndicatorsOn;

            FillInnCbChSubstances();
            FillInntxtLogFolder();
            SelectPreviousCbChSubstances();
            COM.ConnectToArduino();

        }
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            Settings.WriteKnownSubstancesToFile();
            Settings.WritePreviousLogFolder();
            Settings.WriteSelectedChSubstances(cbSubstanceList);
        }
        private void FillInnCbChSubstances()
        {
            Settings.ReadKnownSubstancesFromFile();
            UpdateCbChSubstances();
        }
        private void FillInntxtLogFolder()
        {
            string buffer = Settings.ReadPreviousLogFolder();
            if (buffer != null && buffer != "")
            {
                Log.LogFilePathFolder = buffer;
                txtLogFolder.Text = buffer;
            }
        }
        private void SelectPreviousCbChSubstances()
        {
            List<string> selectedSubstances = Settings.ReadSelectedChSubstances();
            for (int i = 0; i < selectedSubstances.Count; i++)
            {
                cbSubstanceList[i].SelectedItem = selectedSubstances[i];
            }
        }
        private void btnSearch_Click(object sender, EventArgs e)
        {
            tvLog.Nodes.Clear();

            if (dtpFrom.Checked && dtpTo.Checked) Log.FindLogDataFromFile(dtpFrom.Value, dtpTo.Value);
            else if (dtpFrom.Checked) Log.FindLogDataFromFile(dtpFrom.Value, true);
            else if (dtpTo.Checked) Log.FindLogDataFromFile(dtpTo.Value, false);
            else Log.FindLogDataFromFile();

            string logFolder;
            for (int i = 0; i < Log.LogDataList.Count; i++)
            {
                logFolder = Log.LogDataList[i].LogFolder;

                if (!tvLog.Nodes.ContainsKey(logFolder))  // Create parent if it doesnt exist
                {
                    tvLog.Nodes.Add(logFolder, logFolder); // Adding node with the same "Name" and "Text"
                    TreeNode child = new TreeNode(Log.LogDataList[i].TimeAndDate.ToString());
                    child.Tag = Log.LogDataList[i];
                    tvLog.Nodes[tvLog.Nodes.Count - 1].Nodes.Add(child);
                }
                else
                {
                    TreeNode child = new TreeNode(Log.LogDataList[i].TimeAndDate.ToString());
                    child.Tag = Log.LogDataList[i];
                    tvLog.Nodes[tvLog.Nodes.Count - 1].Nodes.Add(child);
                }
            }
        }

        private void tvLog_AfterSelect(object sender, TreeViewEventArgs e)
        {
            lvLog.Items.Clear();
            TreeNode selectedNode = tvLog.SelectedNode;
            if (selectedNode != null && selectedNode.Tag != null)
            {
                LogData nodeLog = (LogData)selectedNode.Tag;

                for (int i = 0; i < nodeLog.ActionArray.Length; i++)
                {
                    string[] data =
                    {
                    nodeLog.ActionArray[i].ChannelNum.ToString(),
                    nodeLog.ActionArray[i].Substance,
                    (nodeLog.ActionArray[i].Duration / 1000.0).ToString()
                    };

                    ListViewItem item = new ListViewItem(data);
                    lvLog.Items.Add(item);
                }

                txtLog.Text = nodeLog.Comment;
            }
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            Channel ch = (Channel)cbChannelSeq.SelectedItem;
            actionList.Add(new PAction(ch.Substance, ch.ChannelNum, (int)(nudTime.Value * 1000)));

            UpdatelvActions();
            //lvActions.Items[0].BackColor = Color.FromArgb(255, 0, 0);
        }

        private void btnStartSequence_Click(object sender, EventArgs e)
        {
            DialogResult runSequence = new DialogResult();
            if (Log.LogFilePathFolder == null)
            {
                runSequence = MessageBox.Show("No log folder selected, unable to log sequence.\nAre you sure you want to run the sequence?",
                                                           "No log folder",
                                                           MessageBoxButtons.YesNo,
                                                           MessageBoxIcon.Warning);
            }
            if (runSequence == DialogResult.Yes)
            {
                currentSequence = new Sequence(actionList, txtComment.Text, "");
                currentSequence.StartSequenceThread(false);
            }
            if (Log.LogFilePathFolder != null)
            {
                currentSequence = new Sequence(actionList, txtComment.Text, "");
                //currentSequence.ActionCompleted += this.Test;
                currentSequence.StartSequenceThread(true);
            }
        }

        private void btnRemove_Click(object sender, EventArgs e)
        {
            actionList.RemoveAt(lvActions.SelectedIndices[0]);
            UpdatelvActions();
        }

        private void UpdatelvActions()
        {
            lvActions.Items.Clear();

            for (int i = 0; i < actionList.Count; i++)
            {
                string[] aStr =
                {
                    actionList[i].ChannelNum.ToString(),
                    actionList[i].Substance,
                    (actionList[i].Duration / 1000.0).ToString()
                };
                ListViewItem item = new ListViewItem(aStr);
                lvActions.Items.Add(item);
            }

        }

        private void btnConnect_Click(object sender, EventArgs e)
        {
            if (COM.ArduinoPort == null || (COM.ArduinoPort != null && !COM.ArduinoPort.IsOpen))
            {
                COM.ConnectToArduino();
            }
        }

        private void btnStopSequence_Click(object sender, EventArgs e)
        {
            currentSequence.StopSequence();
        }

        private void cbSubstance_Leave(object sender, EventArgs e)
        {
            UpdateSequenceGUI();
            UpdateManualGUI();
        }

        private void UpdateManualGUI()
        {
            UpdateManualChannelCB();
        }

        private void UpdateSequenceGUI()
        {
            UpdateActionListNames();
            UpdatelvActions();
            UpdateSequenceChannelCB();
        }

        private void UpdateChannelLists()
        {
            seqChannels.Clear();
            manChannels.Clear();
            seqChannels.Add(new Channel(0, "Wait"));
            for (int i = 0; i < cbSubstanceList.Count; i++)
            {
                if (cbSubstanceList[i].Text != "")
                {
                    seqChannels.Add(new Channel(i + 1, cbSubstanceList[i].Text));
                    manChannels.Add(new Channel(i + 1, cbSubstanceList[i].Text));
                }

            }
        }

        private void UpdateActionListNames()
        {
            string substanceInCB;
            for (int i = 0; i < actionList.Count; i++)
            {
                if (actionList[i].ChannelNum != 0)
                {
                    substanceInCB = cbSubstanceList[actionList[i].ChannelNum - 1].Text;
                }
                else
                {
                    substanceInCB = "Wait";
                }

                if (actionList[i].Substance != substanceInCB)
                {
                    actionList[i] = new PAction(substanceInCB, actionList[i].ChannelNum, actionList[i].Duration);
                }
            }
        }

        private void UpdateSequenceChannelCB()
        {
            UpdateChannelLists();

            cbChannelSeq.Items.Clear();
            for (int i = 0; i < seqChannels.Count; i++)
            {
                cbChannelSeq.Items.Add(seqChannels[i]);
            }

            // Assuming there always is at least one element in cbChannel
            if (cbChannelSeq.SelectedItem == null) cbChannelSeq.SelectedIndex = 0;
        }

        private void UpdateManualChannelCB()
        {
            UpdateChannelLists();

            cbChannelMan.Items.Clear();
            for (int i = 1; i < seqChannels.Count; i++)
            {
                cbChannelMan.Items.Add(seqChannels[i]);
            }

            if (cbChannelMan.Items.Count != 0) cbChannelMan.SelectedIndex = 0;
        }

        private void connectionCheckTimer_Tick(object sender, EventArgs e)
        {
            if (COM.ArduinoPort != null && !COM.ArduinoPort.IsOpen) ConnectionIndicatorsOff();
        }

        private void IncrementActionLVHighlight(object sender, int currentIndex)
        {
            for (int i = 0; i < lvActions.Items.Count; i++)
            {
                if (currentIndex < i) lvActions.Items[i].BackColor = Color.Green;
                else if (currentIndex == i) lvActions.Items[i].BackColor = Color.Blue;
            }
        }

        //private void IncrementActionLVHighlight(int currentIndex)
        //{
        //    for (int i = 0; i < lvActions.Items.Count; i++)
        //    {
        //        if (currentIndex > i) lvActions.Items[i].BackColor = Color.Green;
        //        else if (currentIndex == i) lvActions.Items[i].BackColor = Color.Blue;
        //    }
        //}
        //private void Test(object sender, int currentIndex)
        //{
        //    Del e = new Del(IncrementActionLVHighlight);
        //    Invoke(e, currentIndex);
        //}

        public void ResetActionLVHighlight()
        {
            for (int i = 0; i < lvActions.Items.Count; i++)
            {
                lvActions.Items[i].BackColor = ListView.DefaultBackColor;
            }
        }

        private void StopActionLVHighlight(object sender, EventArgs e)
        {
            int currentIndex = 0;
            lvActions.Items[currentIndex].BackColor = Color.Red;
        }

        private void ConnectionIndicatorsOn()
        {
            pbArduinoStatusSeq.Image = Image.FromFile(AppDomain.CurrentDomain.BaseDirectory + "arduino-logo-blue.png");
            pbArduinoStatusMan.Image = Image.FromFile(AppDomain.CurrentDomain.BaseDirectory + "arduino-logo-blue.png");
            pbArduinoStatusSet.Image = Image.FromFile(AppDomain.CurrentDomain.BaseDirectory + "arduino-logo-blue.png");
        }
        private void ConnectionIndicatorsOff()
        {
            pbArduinoStatusSeq.Image = Image.FromFile(AppDomain.CurrentDomain.BaseDirectory + "arduino-logo-red.png");
            pbArduinoStatusMan.Image = Image.FromFile(AppDomain.CurrentDomain.BaseDirectory + "arduino-logo-red.png");
            pbArduinoStatusSet.Image = Image.FromFile(AppDomain.CurrentDomain.BaseDirectory + "arduino-logo-red.png");
        }

        private void btnOpen_Click(object sender, EventArgs e)
        {
            Channel ch = (Channel)cbChannelMan.SelectedItem;
            Manual.Open(ch.ChannelNum);
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            Channel ch = (Channel)cbChannelMan.SelectedItem;
            Manual.Close(ch.ChannelNum);
        }

        private void btnOpenNext_Click(object sender, EventArgs e)
        {
            Manual.OpenNext(manChannels, cbChannelMan.SelectedIndex);
            // If not at highest index, then increment
            if (cbChannelMan.Items.Count - 1 != cbChannelMan.SelectedIndex) cbChannelMan.SelectedIndex++;
        }

        private void btnSelectFolder_Click(object sender, EventArgs e)
        {
            CommonOpenFileDialog dialog = new CommonOpenFileDialog();
            dialog.IsFolderPicker = true;
            if (dialog.ShowDialog() == CommonFileDialogResult.Ok)
            {
                txtLogFolder.Text = dialog.FileName;
                Log.LogFilePathFolder = dialog.FileName;
            }
        }

        private void btnRemoveSubstance_Click(object sender, EventArgs e)
        {
            Settings.KnownSubstancesList.Remove(cbRemoveSubstance.Text);
            UpdateCbChSubstances();
        }
        private void UpdateCbChSubstances()
        {
            cbRemoveSubstance.Items.Clear();
            for (int i = 0; i < cbSubstanceList.Count; i++)
            {
                cbSubstanceList[i].Items.Clear();
            }
            for (int i = 0; i < Settings.KnownSubstancesList.Count; i++)
            {
                for (int j = 0; j < cbSubstanceList.Count; j++)
                {

                    cbSubstanceList[j].Items.Add(Settings.KnownSubstancesList[i]);
                }
                cbRemoveSubstance.Items.Add(Settings.KnownSubstancesList[i]);
            }

            for (int i = 0; i < cbSubstanceList.Count; i++)
            {
                if (cbSubstanceList[i].SelectedItem == null)
                {
                    if (Settings.KnownSubstancesList.Count != 0)
                    {
                        cbSubstanceList[i].SelectedIndex = 0;
                    }
                    else
                    {
                        cbSubstanceList[i].Text = "";
                    }
                }
            }
            UpdateSequenceGUI();
        }
    }
}
