﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Text;
using System.Threading.Tasks;
using System.IO;

namespace Perfusjonssytem
{
    enum Result { Completed, Stoped, Disconnected }
    class LogData
    {
        bool valid = true;

        DateTime timeAndDate;
        PAction[] actionArray;
        Result result;
        string comment;

        List<string> folders;

        public LogData(DateTime timeAndDate_, List<PAction> actionList_, Result result_, string comment_)
        {
            folders = new List<string>();

            int length = actionList_.Count;

            timeAndDate = timeAndDate_;
            actionArray = new PAction[length];
            actionList_.CopyTo(actionArray); 
            result = result_;
            comment = comment_;
        }

        public PAction[] ActionArray { get => actionArray; }
        public string Comment { get => comment; set => comment = value; }
        public DateTime TimeAndDate { get => timeAndDate; }
        public List<string> Folders { get => folders; set => folders = value; }
        public bool Valid { get => valid; set => valid = value; }
        internal Result Result { get => result; }
    }


    static class Log
    {
        static string logFilePathFolder;
        static List<LogData> logDataList = new List<LogData>();

        public static string LogFilePathFolder { get => logFilePathFolder; set => logFilePathFolder = value; }
        public static List<LogData> LogDataList { get => logDataList; }

        static public void SaveToFile(LogData logToSave)
        {
            string fileName = logToSave.TimeAndDate.ToString("dd.MM.yyyy HH.mm.ss");
            string directoryString = string.Format(@logFilePathFolder + "\\" + fileName + ".txt");
            StreamWriter streamWriter = new StreamWriter(@directoryString);

            streamWriter.WriteLine(logToSave.TimeAndDate.ToString("dd.MM.yyyy HH.mm.ss"));
            for (int i = 0; i < logToSave.ActionArray.Length; i++)
            {
                streamWriter.WriteLine(logToSave.ActionArray[i].Substance);
                streamWriter.WriteLine(logToSave.ActionArray[i].ChannelNum);
                streamWriter.WriteLine(logToSave.ActionArray[i].Duration);
            }
            streamWriter.WriteLine(); // Indicates end of actionList when reading from file
            streamWriter.WriteLine(logToSave.Result);
            streamWriter.Write(logToSave.Comment);
            streamWriter.Close();
        }

        static public LogData ReadFromFile(string logFilePath)
        {
            StreamReader sr = new StreamReader(logFilePath);

            DateTime timeAndDate = new DateTime();
            List<PAction> actionList = new List<PAction>();
            string substance;
            int channel;
            int duration;

            Result result;
            string comment = "";

            // Finds log folder from file path
            int index = logFilePath.LastIndexOf("\\");
            string logFolder = logFilePath.Remove(index);
            index = logFolder.LastIndexOf("\\");
            logFolder = logFolder.Remove(0, index + 1);
            // ***********

            try
            {
                string test = sr.ReadLine();
                timeAndDate = DateTime.ParseExact(test, "dd.MM.yyyy HH.mm.ss", CultureInfo.InvariantCulture);

                // Read all actions to actionList
                while (true)
                {
                    substance = sr.ReadLine();
                    if (substance == "") break;
                    channel = Convert.ToInt32(sr.ReadLine());
                    duration = Convert.ToInt32(sr.ReadLine());
                    actionList.Add(new PAction(substance, channel, duration));
                }

                string resultBuffer = sr.ReadLine();
                Enum.TryParse(resultBuffer, true, out result);

                comment = sr.ReadToEnd();
            }
            catch (Exception)
            {
                LogData invalidData = new LogData(DateTime.Now, new List<PAction>(), Result.Stoped, "");
                invalidData.Valid = false;
                return invalidData;
            }

            return new LogData(timeAndDate, actionList, result, comment);
        }

        static public void FindLogDataFromFile(DateTime from, DateTime to)
        {
            logDataList.Clear();
            string[] logFilePaths = Directory.GetFiles(AppDomain.CurrentDomain.BaseDirectory + "Logs\\", "*.txt", SearchOption.AllDirectories);


            int dateStartIndex;
            int dateStopIndex;
            string timeAndDateStr;
            DateTime timeAndDate;

            for (int i = 0; i < logFilePaths.Length; i++)
            {
                dateStartIndex = logFilePaths[i].LastIndexOf("\\") + 1;   // +1 to not include "/" in the string
                dateStopIndex = logFilePaths[i].LastIndexOf('.');
                timeAndDateStr = logFilePaths[i].Substring(dateStartIndex, dateStopIndex - dateStartIndex);
                timeAndDate = DateTime.ParseExact(timeAndDateStr, "dd.MM.yyyy HH.mm.ss", CultureInfo.InvariantCulture);
                if (timeAndDate >= from && timeAndDate <= to)
                {
                    LogData buffer = ReadFromFile(logFilePaths[i]);
                    if (buffer.Valid)
                    {
                        logDataList.Add(buffer);
                        logDataList[logDataList.Count - 1].Folders = CreateFolderList(logFilePaths[i]);
                    }
                }
            }
        }

        static public void FindLogDataFromFile(DateTime searchDate, bool from)
        {
            logDataList.Clear();
            string[] logFilePaths = Directory.GetFiles(AppDomain.CurrentDomain.BaseDirectory + "Logs\\", "*.txt", SearchOption.AllDirectories);


            int dateStartIndex;
            int dateStopIndex;
            string timeAndDateStr;
            DateTime timeAndDate;

            for (int i = 0; i < logFilePaths.Length; i++)
            {
                dateStartIndex = logFilePaths[i].LastIndexOf("\\") + 1;   // +1 to not include "/" in the string
                dateStopIndex = logFilePaths[i].LastIndexOf('.');
                timeAndDateStr = logFilePaths[i].Substring(dateStartIndex, dateStopIndex - dateStartIndex);
                timeAndDate = DateTime.ParseExact(timeAndDateStr, "dd.MM.yyyy HH.mm.ss", CultureInfo.InvariantCulture);
                if (from)
                {
                    if (timeAndDate >= searchDate)
                    {
                        LogData buffer = ReadFromFile(logFilePaths[i]);
                        if (buffer.Valid)
                        {
                            logDataList.Add(buffer);
                            logDataList[logDataList.Count - 1].Folders = CreateFolderList(logFilePaths[i]);
                        }
                    }
                }
                else
                {
                    if (timeAndDate <= searchDate)
                    {
                        LogData buffer = ReadFromFile(logFilePaths[i]);
                        if (buffer.Valid)
                        {
                            logDataList.Add(buffer);
                            logDataList[logDataList.Count - 1].Folders = CreateFolderList(logFilePaths[i]);
                        }
                    }
                }
            }
        }

        static public void FindLogDataFromFile()
        {
            logDataList.Clear();
            string[] logFilePaths = Directory.GetFiles(AppDomain.CurrentDomain.BaseDirectory + "Logs\\", "*.txt", SearchOption.AllDirectories);

            for (int i = 0; i < logFilePaths.Length; i++)
            {
                LogData buffer = ReadFromFile(logFilePaths[i]);
                if (buffer.Valid)
                {
                    logDataList.Add(buffer);
                    logDataList[logDataList.Count - 1].Folders = CreateFolderList(logFilePaths[i]);
                }
            }
        }

        private static List<string> CreateFolderList(string filePath)
        {
            List<string> folders = new List<string>();
            string shortPath = filePath.Substring(filePath.IndexOf("\\Logs") + 6);
            string folder;

            while (shortPath.Contains("\\"))
            {
                folder = shortPath.Substring(0, shortPath.IndexOf("\\"));
                shortPath = shortPath.Remove(0, shortPath.IndexOf("\\") + 1);
                folders.Add(folder);
            }

            return folders;
        }
    }
}
