﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using System.Diagnostics;

namespace Perfusjonssytem
{
    public partial class PerfusionForm : Form
    {
        SequenceForm sequenceForm;
        ManualForm manualForm;
        LogForm logForm;
        SettingsForm settingsForm;
        Stopwatch seqDurationSW;

        static string imageDerectory = AppDomain.CurrentDomain.BaseDirectory + "Images\\";

        bool exiting;
        object exitingSync;
        Color menuButtonColor;

        public bool Exiting { get => exiting; set => exiting = value; }
        public object ExitingSync { get => exitingSync; set => exitingSync = value; }
        public SeqIndicator SeqIndicator { get => seqIndicator; }
        public SequenceForm SequenceForm { get => sequenceForm; set => sequenceForm = value; }
        public ManualForm ManualForm { get => manualForm; set => manualForm = value; }
        public LogForm LogForm { get => logForm; set => logForm = value; }
        public SettingsForm SettingsForm { get => settingsForm; set => settingsForm = value; }
        public Button BtnSequence { get => btnSequence; set => btnSequence = value; }
        public Button BtnManual { get => btnManual; set => btnManual = value; }
        public Button BtnSettings { get => btnSettings; set => btnSettings = value; }
        public Stopwatch SeqDurationSW { get => seqDurationSW; set => seqDurationSW = value; }

        public PerfusionForm()
        {
            InitializeComponent();

            settingsForm = new SettingsForm(this);
            sequenceForm = new SequenceForm(this);
            manualForm = new ManualForm(this);
            logForm = new LogForm();

            seqDurationSW = new Stopwatch();

            childFormPanel.Controls.Add(sequenceForm);
            childFormPanel.Controls.Add(manualForm);
            childFormPanel.Controls.Add(logForm);
            childFormPanel.Controls.Add(settingsForm);
        }

        // STARTUP //******************************************************
        #region STARTUP 
        private void PerfusionForm_Load(object sender, EventArgs e)
        {
            COM.PerfusionFormCOM = this;

            Exiting = false;
            ExitingSync = new object();

            COM.ConnectedDel += ConnectionIndicatorsOn;
            COM.ConnectedDel += ManualForm.CloseAll;
            COM.DisconnectedDel += ConnectionIndicatorsOff;


            COM.ConnectToArduino();

            menuButtonColor = btnSequence.BackColor;
            btnSequence_Click(this, EventArgs.Empty);
        }

        #endregion
        //*****************************************************************

        // BUTTON EVENTS //************************************************
        #region BUTTON EVENTS

        private void btnSequence_Click(object sender, EventArgs e)
        {
            foreach (Control c in childFormPanel.Controls)
            {
                c.Hide();
            }
            sequenceForm.Show();
            btnSequence.BackColor = btnSequence.FlatAppearance.MouseDownBackColor;
            btnManual.BackColor = menuButtonColor;
            btnLog.BackColor = menuButtonColor;
            btnSettings.BackColor = menuButtonColor;
        }

        private void btnManual_Click(object sender, EventArgs e)
        {
            foreach (Control c in childFormPanel.Controls)
            {
                c.Hide();
            }
            manualForm.Show();
            btnManual.BackColor = btnManual.FlatAppearance.MouseDownBackColor;
            btnSequence.BackColor = menuButtonColor;
            btnLog.BackColor = menuButtonColor;
            btnSettings.BackColor = menuButtonColor;
        }

        private void btnLog_Click(object sender, EventArgs e)
        {
            foreach (Control c in childFormPanel.Controls)
            {
                c.Hide();
            }
            logForm.Show();
            btnLog.BackColor = btnLog.FlatAppearance.MouseDownBackColor;
            btnSequence.BackColor = menuButtonColor;
            btnManual.BackColor = menuButtonColor;
            btnSettings.BackColor = menuButtonColor;
        }

        private void btnSettings_Click(object sender, EventArgs e)
        {
            foreach (Control c in childFormPanel.Controls)
            {
                c.Hide();
            }
            settingsForm.Show();
            btnSettings.BackColor = btnSettings.FlatAppearance.MouseDownBackColor;
            btnSequence.BackColor = menuButtonColor;
            btnLog.BackColor = menuButtonColor;
            btnManual.BackColor = menuButtonColor;
        }
        #endregion
        //*****************************************************************

        // OTHER EVENTS //*************************************************
        #region OTHER EVENTS

        #endregion
        //*****************************************************************

        // UPDATES //******************************************************
        #region UPDATES

        private void ConnectionIndicatorsOn()
        {
            sequenceForm.PbArduinoStatusSeq.Image.Dispose();
            manualForm.PbArduinoStatusMan.Image.Dispose();
            settingsForm.PbArduinoStatusSet.Image.Dispose();
            sequenceForm.PbArduinoStatusSeq.Image = Image.FromFile(imageDerectory + "arduino-logo-blue.png");
            manualForm.PbArduinoStatusMan.Image = Image.FromFile(imageDerectory + "arduino-logo-blue.png");
            settingsForm.PbArduinoStatusSet.Image = Image.FromFile(imageDerectory + "arduino-logo-blue.png");
        }
        private void ConnectionIndicatorsOff()
        {
            sequenceForm.PbArduinoStatusSeq.Image.Dispose();
            manualForm.PbArduinoStatusMan.Image.Dispose();
            settingsForm.PbArduinoStatusSet.Image.Dispose();
            sequenceForm.PbArduinoStatusSeq.Image = Image.FromFile(imageDerectory + "arduino-logo-redV2.png");
            manualForm.PbArduinoStatusMan.Image = Image.FromFile(imageDerectory + "arduino-logo-redV2.png");
            settingsForm.PbArduinoStatusSet.Image = Image.FromFile(imageDerectory + "arduino-logo-redV2.png");
        }

        bool draggingWindow = false;
        Point offset;
        private void dragPanel_MouseDown(object sender, MouseEventArgs e)
        {
            draggingWindow = true;
            offset = Point.Subtract(Cursor.Position, new Size(this.Location));
        }
        private void dragPanel_MouseUp(object sender, MouseEventArgs e)
        {
            draggingWindow = false;
        }
        private void dragpanel_MouseMove(object sender, MouseEventArgs e)
        {
            if (draggingWindow)
            {
                this.Location = Point.Subtract(Cursor.Position, new Size(offset));
            }
        }

        private void seqDurationTimer_Tick(object sender, EventArgs e)
        {
            TimeSpan ts = seqDurationSW.Elapsed;
            string elapsedTime = String.Format("{0:0}:{1:00}:{2:00}.{3:0}",
            ts.Hours, ts.Minutes, ts.Seconds,
            ts.Milliseconds / 100);
            lblDuration.Text = elapsedTime;
        }

        public void StopSequenceDuration()
        {
            seqDurationTimer.Enabled = false;
            seqDurationSW.Stop();
            seqDurationTimer_Tick(new object(), EventArgs.Empty);
        }
        #endregion
        //*****************************************************************

        // STUT DOWN //****************************************************
        #region SHUTDOWN
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            lock (exitingSync)
            {
                Exiting = true;
            }

            Settings.WriteKnownSubstancesToFile();
            Settings.WritePreviousLogFolder();
            Settings.WriteSelectedChSubstances(settingsForm.CbSubstanceList);
            Settings.WritePreviousSequence(sequenceForm.ActionList);

            if (sequenceForm.CurrentSequence != null)
            {
                bool sequenceRunning;
                lock (sequenceForm.CurrentSequence.runningSync)
                {
                    sequenceRunning = sequenceForm.CurrentSequence.Running;
                }
                if (sequenceRunning)
                {
                    seqIndicator.Stop();
                    StopSequenceDuration();
                    sequenceForm.CurrentSequence.StopSequence();
                    sequenceForm.CurrentSequence.WaitForSequenceThread();
                }
            }

            //Manual.CloseAll();
        }

        private void pictureBox1_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        #endregion
        //*****************************************************************
    }
}
