﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.IO;
using System.Threading.Tasks;
using System.ComponentModel;

namespace Perfusjonssytem
{
    static class Settings
    {
        private static BindingList<string> knownSubstancesList = new BindingList<string>();
        static private string folderName = "Settings";
        static private string knownSubstancesFileName = "KnownSubstances.txt";
        static private string previousLogFolderName = "PreviousLogFolderName.txt";
        static private string selectedChSubstancesFileName = "SelectedChSubstances.txt";
        static private string previousSequenceFolderName = "PreviousSequence.txt";

        public static BindingList<string> KnownSubstancesList { get => knownSubstancesList; set => knownSubstancesList = value; }


        static public void ReadKnownSubstancesFromFile()
        {
            StreamReader streamReader = new StreamReader(AppDomain.CurrentDomain.BaseDirectory + "\\" + folderName + "\\" + knownSubstancesFileName);
            while (!streamReader.EndOfStream)
            {
                string substanceFromFile = streamReader.ReadLine();
                knownSubstancesList.Add(substanceFromFile);
            }
            streamReader.Close();
            if (!knownSubstancesList.Contains("None"))
            {
                knownSubstancesList.Insert(0,"None");
            }
        }
        static public void WriteKnownSubstancesToFile()
        {
            StreamWriter streamWriter = new StreamWriter(AppDomain.CurrentDomain.BaseDirectory + "\\" + folderName + "\\" + knownSubstancesFileName);
            for (int i = 0; i < knownSubstancesList.Count; i++)
            {
                if (i != knownSubstancesList.Count-1) streamWriter.WriteLine(knownSubstancesList[i]);
                else streamWriter.Write(knownSubstancesList[i]);
            }
            streamWriter.Close();
        }
        static public string ReadPreviousLogFolder()
        {
            StreamReader streamReader = new StreamReader(AppDomain.CurrentDomain.BaseDirectory + "\\" + folderName + "\\" + previousLogFolderName);
            string previousLogFolder = streamReader.ReadLine();
            streamReader.Close();
            return previousLogFolder;
        }
        static public void WritePreviousLogFolder()
        {
            StreamWriter streamWriter = new StreamWriter(AppDomain.CurrentDomain.BaseDirectory + "\\" + folderName + "\\" + previousLogFolderName);
            streamWriter.Write(Log.LogFilePathFolder);
            streamWriter.Close();
        }
        static public List<string> ReadSelectedChSubstances()
        {
            StreamReader streamReader = new StreamReader(AppDomain.CurrentDomain.BaseDirectory + "\\" + folderName + "\\" + selectedChSubstancesFileName);
            List<string> chSubstancesList = new List<string>();
            while (!streamReader.EndOfStream)
            {
                string chSubstanceFromFile = streamReader.ReadLine();
                chSubstancesList.Add(chSubstanceFromFile);
            }
            streamReader.Close();
            return chSubstancesList;
        }
        static public void WriteSelectedChSubstances(List<ComboBox> comboBoxList)
        {
            StreamWriter streamWriter = new StreamWriter(AppDomain.CurrentDomain.BaseDirectory + "\\" + folderName + "\\" + selectedChSubstancesFileName);
            for (int i = 0; i < comboBoxList.Count; i++)
            {
                if (i != comboBoxList.Count - 1) streamWriter.WriteLine(comboBoxList[i].SelectedItem);
                else streamWriter.Write(comboBoxList[i].SelectedItem);
            }
            streamWriter.Close();
        }
        static public void ReadPreviousSequence(List<PAction> actionList)
        {
            StreamReader streamReader = new StreamReader(AppDomain.CurrentDomain.BaseDirectory + "\\" + folderName + "\\" + previousSequenceFolderName);
            PAction action;
            while (!streamReader.EndOfStream)
            {
                string substance = streamReader.ReadLine();
                int channel = Convert.ToInt32(streamReader.ReadLine());
                int duration = Convert.ToInt32(streamReader.ReadLine());

                action = new PAction(substance, channel,duration );

                actionList.Add(action);
            }
            streamReader.Close();
        }
        static public void WritePreviousSequence(List<PAction> actionList)
        {
            StreamWriter streamWriter = new StreamWriter(AppDomain.CurrentDomain.BaseDirectory + "\\" + folderName + "\\" + previousSequenceFolderName);
            for (int i = 0; i < actionList.Count; i++)
            {
                streamWriter.WriteLine(actionList[i].Substance);
                streamWriter.WriteLine(actionList[i].ChannelNum);
                streamWriter.WriteLine(actionList[i].Duration);
            }
            streamWriter.Close();
        }

    }
}
